# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Horia Filimonescu horia@filimonescu.de
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import os
import sys
import string
import platform

from angajat import angajatconfig
from angajat.angajatculori import pc as pc
from angajat.angajatculori import pm as pm
from angajat.angajatculori import fs as fs

hf_user=''
hf_name=''
hf_pass=''
hf_grup='public'
hf_grid='59999'

import gettext
from gettext import gettext as _
gettext.textdomain('angajat')

####    Do some checks and return a value

#   Am I running on GNU/Linux?
def i_am_linux():
    if platform.system() == "Linux":
        return True
    else:
        return False

#   Am I running as ROOT?
def i_am_root():
    if os.getuid() != 0:
        return False
    else:
        return True

#   Does the user exist in Linux?
def exists_user():
    import pwd

    global hf_user

    try:
        pwd.getpwnam(hf_user)
        return True
    except:
        return False

#   Does the Public Group exist in Linux?
def exists_group():
    import grp

    global hf_grup

    try:
        grp.getgrnam(hf_grup)
        return True
    except KeyError:
        return False

def exists_home():
    global hf_user

    path='/home/'+hf_user

    if os.path.exists(path):
        return True
    else:
        return False


#   Do the Passwords match and have the required length?
def passwords_match(pass1=None, pass2=None):
    if len(pass1) < 5:
        return False

    if pass1 == pass2:
        return True
    else:
        return False

#   Run a command and return the status of the execution
def run_command(command=None):
    import commands

    if command is None:
        quit(_('Command is empty!'))

    if commands.getstatusoutput(command)[0] == 0:
        return True
    else:
        return False

def list_users():
    import pwd, grp

    print(_('Full Name').center(25)),
    print(_('UID').center(5)),
    print(_('User').center(8))
    print('-'*25),
    print('-'*5),
    print('-'*8)

    for p in pwd.getpwall():
        if p[2] >= 1000 and p[2] <= 2000:
            print(str(p[4])).strip(',').ljust(25),
            print(str(p[2])).rjust(5),
            print(str(p[0])).ljust(8)

####    Read data from the user input

#   What is the Username?
def read_username():
    global hf_user

    hf_user = raw_input(pc(_("Username: "), "bcyan"))

#   What is the Full Name?
def read_fullname():
    global hf_name

    first_name = raw_input(pc(_("First Name: "), "bcyan"))
    last_name = raw_input(pc(_("Last Name: "), "bcyan"))

    hf_name = first_name + ' ' + last_name

#   What is the Password?
def read_password():
    from getpass import getpass

    global hf_pass

    pass1 = getpass(pc(_("New Password: "), "bcyan"), stream=None)
    pass2 = getpass(pc(_("Verify Password: "), "bcyan"), stream=None)

    if passwords_match(pass1, pass2):
        hf_pass = pass1
    else:
        quit(pc(_('Passwords missmach!'), 'bred'))


####    Finally!!!
####    Get the work done

###
#   Create
###

##
#   Create Public Group in GNU/Linux
#
#   [Detailded description]
#
#   @access	private
#   @vars	hf_grup     global
#   @vars	hf_grid     global
#   @return	void
##
def create_group():
    global hf_grup
    global hf_grid

    if exists_group():
        fs('Info',
            _('Group %s already exists.' % pc(hf_grup, 'byellow')))
    else:
		command='groupadd '+hf_grup
		if hf_grid:
			command += ' -g '+hf_grid

        if run_command(command):
            fs('Info',
                _('Group %s created succesfuly.' % pc(hf_grup, 'byellow')))
        else:
            fs('Error',
                _('Group %s not created.' % pc(hf_grup, 'byellow')))

###
#   Create User in GNU/Linux
#
#   [Detailded description]
#
#   @access	private
#   @module crypt
#   @param	hf_user     global
#   @param	hf_name     global
#   @param	hf_grup     global
#   @param	hf_grid     global
#   @param	hf_pass     global
#   @return	void
#
#   @todo   Review description
###

def create_user():
    from crypt import crypt

    global hf_user
    global hf_name
    global hf_grup
    global hf_grid
    global hf_pass

    if exists_user():
        fs('Error',
            _('User %s already exists. Please choose another Username.'
                % pc(hf_user, 'byellow')))
        quit()

    hf_code=crypt(hf_pass, "password")

    comanda = 'useradd '
    comanda += '--create-home '
    comanda += '--user-group '
    comanda += '--groups "'+hf_grup+'" '
    comanda += '--comment "'+hf_name+',,," '
    comanda += '--skel "/dev/null" '
    comanda += '--shell "/bin/false" '
    comanda += '--password "'+hf_code+'" '
    comanda += hf_user

    if run_command(comanda):
        fs('Info',
            _('User %s created succesfuly.' % pc(hf_user, 'byellow')))
    else:
        fs('Error',
            _('User %s could not be created.' % pc(hf_user, 'byellow')))

def create_samba():
    global hf_user
    global hf_pass

    comanda='(echo "'+hf_pass+'"; echo "'+hf_pass+'") | smbpasswd -s -a "'+hf_user+'"'

	if not exists_user():
        fs('Error',
            _("User %s does not exist."
                % pc(hf_user, 'byellow')))
        fs(' --> ', _("You need to add it to Linux in order to use it with Samba."))
        quit()

    if run_command(comanda):
        fs('Info',
            _('User %s added to Samba.' % pc(hf_user, 'byellow')))
    else:
        fs('Error',
            _('User %s could not be added to Samba.' % pc(hf_user, 'byellow')))


###
#   Delete
###

def delete_samba():
    global hf_user

    if not exists_user():
        fs('Error',
            _("User %s does not exist."
                % pc(hf_user, 'byellow')))
        fs(' --> ', _("You cannot delete a user from Samba if it doenen't exist in GNU/Linux."))
        quit()

    comanda='smbpasswd -x '+hf_user

    if run_command(comanda):
        fs('Info',
            _('User %s deleted successfuly from Samba.' % pc(hf_user, 'byellow')))
    else:
        fs('Error',
            _('User %s could not be deleted from Samba.' % pc(hf_user, 'byellow')))

##
#   Delete User from GNU/Linux
##

def delete_user():
    global hf_user

    if not exists_user():
        fs('Error',
            _('User %s doesen\'t. Please select an existing Username.'
                % pc(hf_user, 'byellow')))
        quit()

    comanda = 'userdel '+hf_user

    if run_command(comanda):
        fs('Info',
            _('User %s deleted succesfuly.' % pc(hf_user, 'byellow')))
    else:
        fs('Error',
            _('User %s could not be deleted.' % pc(hf_user, 'byellow')))

def delete_home():
    import shutil

    global hf_user

    if exists_user():
        quit('User %s still exist in Linux!' % hf_user)

    if not exists_home():
        quit('Home folder for user %s does not exist!' % hf_user)

    path='/home/'+hf_user

    try:
        shutil.rmtree(path)
        fs('Info',
            _('Home folder %s deleted successfuly.') %pc(path, 'byellow'))
    except OSError:
        fs('Error',
            _('Home folder %s could not be deleted.') %pc(path, 'byellow'))

def enable_samba():
    global hf_user

    if not exists_user():
        fs('Error',
            _("User %s does not exist."
                % pc(hf_user, 'byellow')))
        fs(' --> ', _("You cannot enable a user in Samba if it doesn't exist in GNU/Linux."))
        quit()

    comanda='smbpasswd -e '+hf_user

    if run_command(comanda):
        fs('Info',
            _('User %s enabled successfuly in Samba.' % pc(hf_user, 'byellow')))
    else:
        fs('Error',
            _('User %s could not be enabled in Samba.' % pc(hf_user, 'byellow')))

def disable_samba():
    global hf_user

    if not exists_user():
        fs('Error',
            _("User %s does not exist."
                % pc(hf_user, 'byellow')))
        fs(' --> ', _("You cannot disable a user in Samba if it doesn't exist in GNU/Linux."))
        quit()

    comanda='smbpasswd -d '+hf_user

    if run_command(comanda):
        fs('Info',
            _('User %s disabled successfuly in Samba.' % pc(hf_user, 'byellow')))
    else:
        fs('Error',
            _('User %s could not be disabled in Samba.' % pc(hf_user, 'byellow')))


####    Print program header
def print_hf():
    import socket
    from datetime import date

    print('')
    print('Angajat '+angajatconfig.__version__)
    print('(C) 2012 Horia Filimonescu horia@filimonescu.de')
    print('-' * 70 + '\n')
    fs(_('Host'), socket.gethostname())
    fs(_('Date'), date.today().strftime("%d. %B %Y"))
    print('')
    fs(_('Python'), str(platform.python_version()))
    print('')
